package JobBulletin::Controller::Jobs;
use Moose;
use namespace::autoclean;

BEGIN { extends 'Catalyst::Controller::HTML::FormFu'; }

=head1 NAME

JobBulletin::Controller::Jobs - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut

=head2 index

=cut

sub index :Path :Args(0) {
    my ($self, $c) = @_;
    $c->response->body('Matched JobBulletin::Controller::Jobs in Jobs.');
}

=head2 list

Fetch all job objects and pass to jobs/list.tt2 in stash to be displayed

=cut

sub list :Local {
    my ($self, $c) = @_;
    $c->stash(jobs => [$c->model('DB::Job')->all]);
    $c->stash(template => 'jobs/list.tt2');
}

=head2 base

Can place common logic to start chained dispatch here

=cut

sub base :Chained('/') :PathPart('jobs') :CaptureArgs(0) {
    my ($self, $c) = @_;
    $c->stash(resultset => $c->model('DB::Job'));
    }

=head2 object

Fetch the specified job object based on the job id and store
it in the stash

=cut

sub object :Chained('base') :PathPart('id') :CaptureArgs(1) {
    my ($self, $c, $id) = @_;
    $c->stash(object => $c->stash->{resultset}->find($id));
    die "Job $id not found!" if !$c->stash->{object};
}

=head2 delete

Delete a job

=cut

sub delete :Chained('object') :PathPart('delete') :Args(0) {
    my ($self, $c) = @_;
    $c->stash->{object}->delete;
    $c->response->redirect($c->uri_for($self->action_for('list'),{status_msg => "Job deleted."}));
}

=head2 formfu_create

Use HTML::FormFu to create a new job

=cut

sub formfu_create :Chained('base') :PathPart('formfu_create') :Args(0) :FormConfig {
    my ($self, $c) = @_;

    my $form = $c->stash->{form};
    if ($form->submitted_and_valid) {
        my $job = $c->model('DB::Job')->new_result({});
        $form->model->update($job);
        $c->response->redirect($c->uri_for($self->action_for('list'), {status_msg => "Job created."}));
        $c->detach;
    } else {
        my @category_objs = $c->model("DB::Category")->all();
        my @categories;
        foreach (sort {$a->title cmp $b->title} @category_objs) {
            push(@categories, [$_->id, $_->title]);
        }
        my $select = $form->get_element({type => 'Select'});
        $select->options(\@categories);
    }
    $c->stash(template => 'jobs/formfu_create.tt2');
}

=head2 formfu_edit

Use HTML::FormFu to update an existing job

=cut

sub formfu_edit :Chained('object') :PathPart('formfu_edit') :Args(0)
        :FormConfig('jobs/formfu_create.yml') {
    my ($self, $c) = @_;

    my $job = $c->stash->{object};
    unless ($job) {
        $c->response->redirect($c->uri_for($self->action_for('list'), {error_msg => "Invalid job -- cannot edit."}));
        $c->detach;
    }

    my $form = $c->stash->{form};
    if ($form->submitted_and_valid) {
        $form->model->update($job);
        $c->response->redirect($c->uri_for($self->action_for('list'), { status_msg => "Job edited"}));
        $c->detach;
    } else {
        my @category_objs = $c->model("DB::Category")->all();
        my @categories;
        foreach (sort {$a->title cmp $b->title} @category_objs) {
            push(@categories, [$_->id, $_->title]);
        }
        my $select = $form->get_element({type => 'Select'});
        $select->options(\@categories);
        $form->model->default_values($job);
    }
    $c->stash(template => 'jobs/formfu_create.tt2');
}

=head1 AUTHOR

A clever guy

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
